@@ -18,7 +18,8 @@ pp = pprint.PrettyPrinter(width=120)
 sonos_max_items = 66000
 # ToDo: Understand why a hard stop is required to exit 'wait_stopped_for'
 #       and revert the variable below. Events related?
-hard_stop = False
+use_sigkill = False
+
 
 # Error handling functions 3.7
 def error_and_exit(msg):
@@ -94,6 +95,17 @@ def two_parameters(f):
     return wrapper
 
 
+def one_or_more_parameters(f):
+    def wrapper(*args, **kwargs):
+        if len(args[2]) < 1:
+            parameter_number_error(args[1], "1 or more")
+            return False
+        else:
+            return f(*args, **kwargs)
+
+    return wrapper
+
+
 # Utility functions
 def seconds_until(time_str):
     # target_time = datetime.time.fromisoformat(time_str)
@@ -161,6 +173,14 @@ def convert_true_false(true_or_false, conversion="YesOrNo"):
         return "on" if true_or_false is True else "off"
 
 
+def print_playlist_header(playlist_name):
+    spacer = "  "
+    title = "Sonos Playlist: {}".format(playlist_name)
+    underline = "=" * (len(title))
+    print(spacer + title)
+    print(spacer + underline)
+
+
 def print_tracks(tracks):
     item_number = 1
     for track in tracks:
@@ -177,7 +197,7 @@ def print_tracks(tracks):
         except:
             title = ""
         print(
-            "{:5d}: Artist: {} | Album: {} | Title: {}".format(
+            "{:7d}: Artist: {} | Album: {} | Title: {}".format(
                 item_number, artist, album, title
             )
         )
@@ -633,11 +653,13 @@ def playlist_operations(speaker, action, args, soco_function, use_local_speaker_
 def list_playlist_tracks(speaker, action, args, soco_function, use_local_speaker_list):
     playlist = get_playlist(speaker, args[0])
     if playlist:
-        print("Sonos Playlist: {}".format(playlist.title))
+        print()
+        print_playlist_header(playlist.title)
         tracks = speaker.music_library.browse_by_idstring(
             "sonos_playlists", playlist.item_id, max_items=sonos_max_items
         )
         print_tracks(tracks)
+        print()
         return True
     else:
         error_and_exit("Playlist '{}' not found".format(args[0]))
@@ -847,8 +869,9 @@ def list_all_playlist_tracks(
     speaker, action, args, soco_function, use_local_speaker_list
 ):
     playlists = speaker.get_sonos_playlists(complete_result=True)
+    print()
     for playlist in playlists:
-        print("Sonos Playlist: {}".format(playlist.title))
+        print_playlist_header(playlist.title)
         tracks = speaker.music_library.browse_by_idstring(
             "sonos_playlists", playlist.item_id
         )
@@ -859,7 +882,10 @@ def list_all_playlist_tracks(
 
 @zero_parameters
 def wait_stop(speaker, action, args, soco_function, use_local_speaker_list):
-    sub = speaker.avTransport.subscribe(auto_renew=True)
+    try:
+        sub = speaker.avTransport.subscribe(auto_renew=True)
+    except Exception as e:
+        error_and_exit("Exception {}".format(e))
     while True:
         try:
             event = sub.events.get(timeout=1.0)
@@ -877,18 +903,25 @@ def wait_stop(speaker, action, args, soco_function, use_local_speaker_list):
 
 @one_parameter
 def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_list):
+    global use_sigkill
     duration = sonos.convert_to_seconds(args[0])
     if not duration:
         parameter_type_error(action, "Time h/m/s or HH:MM:SS")
     logging.info("Waiting until playback stopped for {}s".format(duration))
-    sub = speaker.avTransport.subscribe(auto_renew=True)
+    try:
+        sub = speaker.avTransport.subscribe(auto_renew=True)
+    except Exception as e:
+        error_and_exit("Exception {}".format(e))
     while True:
         try:
+            # ToDo: Remove temporary fix for CTRL-C not exiting
+            use_sigkill = True
             event = sub.events.get(timeout=1.0)
             if event.variables["transport_state"] != "PLAYING":
                 sub.unsubscribe()
-                global hard_stop
-                hard_stop = True
+                # ToDo: Should really return here and do this some other way ...
+                #       this is what's requiring the SIGKILL
+
                 # Poll for changes; count down reset timer
                 # ToDo: Polling is not ideal; should be redesigned using events
                 # ToDO: Use actual timestamps, not accumulated poll_intervals
@@ -914,16 +947,19 @@ def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_lis
                             elapsed, total_elapsed_since_first_state_change
                         )
                     )
-                hard_stop = False
+                use_sigkill = False
                 return True
         except:
-            hard_stop = False
+            use_sigkill = False
             pass
 
 
 @zero_parameters
 def wait_start(speaker, action, args, soco_function, use_local_speaker_list):
-    sub = speaker.avTransport.subscribe(auto_renew=True)
+    try:
+        sub = speaker.avTransport.subscribe(auto_renew=True)
+    except Exception as e:
+        error_and_exit("Exception {}".format(e))
     while True:
         try:
             event = sub.events.get(timeout=1.0)
@@ -939,6 +975,32 @@ def wait_start(speaker, action, args, soco_function, use_local_speaker_list):
             pass
 
 
+@one_or_more_parameters
+def if_stopped_or_playing(speaker, action, args, soco_function, use_local_speaker_list):
+    """Perform the action only if the speaker is currently in the desired playback state
+    """
+    state = speaker.get_current_transport_info()["current_transport_state"]
+    logging.info(
+        "Condition: '{}': Speaker '{}' is in state '{}'".format(
+            action, speaker.player_name, state
+        )
+    )
+    if (state != "PLAYING" and action == "if_playing") or (
+        state == "PLAYING" and action == "if_stopped"
+    ):
+        logging.info("Action suppressed")
+        return True
+    else:
+        action = args[0]
+        args = args[1:]
+        logging.info(
+            "Action invoked: '{} {} {}'".format(
+                speaker.player_name, action, " ".join(args)
+            )
+        )
+        return process_action(speaker, action, args, use_local_speaker_list)
+
+
 def process_action(speaker, action, args, use_local_speaker_list):
     sonos_function = actions.get(action, None)
     if sonos_function:
@@ -969,6 +1031,7 @@ actions = {
     "dialogue_mode": SonosFunction(on_off_action, "dialog_mode"),
     "dialogue": SonosFunction(on_off_action, "dialog_mode"),
     "play": SonosFunction(no_args_no_output, "play"),
+    "start": SonosFunction(no_args_no_output, "play"),
     "stop": SonosFunction(no_args_no_output, "stop"),
     "pause": SonosFunction(no_args_no_output, "pause"),
     "next": SonosFunction(no_args_no_output, "next"),
@@ -1098,4 +1161,6 @@ actions = {
     "wait_start": SonosFunction(wait_start, ""),
     "wait_stopped_for": SonosFunction(wait_stopped_for, ""),
     "wsf": SonosFunction(wait_stopped_for, ""),
+    "if_stopped": SonosFunction(if_stopped_or_playing, ""),
+    "if_playing": SonosFunction(if_stopped_or_playing, ""),
 }
